/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * A copy of the GNU General Public License is included along with this
 * program in the file gpl-3.0.txt.
 *
 */
Ext.define('Fracking.controller.MapLayers',{
  extend:'Ext.app.Controller',
  views:['MapLayers'],
  init:function() {

    // Show the Surface Water Information panel
    document.getElementById('infoSW').style.display = 'block';
    document.getElementById('infoGW').style.display = 'none';
    document.getElementById('infoVeg').style.display = 'none';
    document.getElementById('infoSei').style.display = 'none';
    document.getElementById('infoSE').style.display = 'none';

    this.control({
      // Handle the selection of variables
      '#idVariable':{change:this.changeVariable},

      // Handle the display of base layers
      '#idRadSW1':{change:this.toggleSWDEC},
      '#idRadSW2':{change:this.toggleSWWetlandRank},
      '#idRadVeg1':{change:this.toggleVegAggr},
      '#idRadVeg2':{change:this.toggleVegETS},
      '#idRadVeg3':{change:this.toggleVegEPL},
      '#idRadSE1':{change:this.toggleSEAggr},
      '#idRadSE2':{change:this.toggleSEDensity},
      '#idRadSE3':{change:this.toggleSEChildren},
      '#idRadSE4':{change:this.toggleSEGroundwater},
      '#idRadSE5':{change:this.toggleSEEmployment},
      '#idRadSE6':{change:this.toggleSEFemaleHH},

      // Handle the display of overlay layers
      '#idChkGW1':{change:this.toggleMining},
      '#idChkGW2':{change:this.togglePASA},
      '#idChkGW3':{change:this.toggleGeolStructures},
      '#idChkGW4':{change:this.toggleGroundwaterCA},
      '#idChkGW5':{change:this.toggleVegter},
      '#idChkGW6':{change:this.toggleRivers},
      '#idChkGW7':{change:this.toggleWMA},
      '#idChkGW8':{change:this.toggleBoreholes},
      '#idChkGW9':{change:this.toggleRoads},
      '#idChkSW1':{change:this.toggleMining},
      '#idChkSW2':{change:this.togglePASA},
      '#idChkSW3':{change:this.toggleSWWetlClusters},
      '#idChkSW4':{change:this.toggleSWSWSABlobs},
      '#idChkSW5':{change:this.toggleSWFish},
      '#idChkSW6':{change:this.toggleRivers},
      '#idChkSW7':{change:this.toggleWMA},
      '#idChkSW8':{change:this.toggleRoads},
      '#idChkVeg1':{change:this.toggleMining},
      '#idChkVeg2':{change:this.togglePASA},
      '#idChkVeg3':{change:this.toggleVegADE},
      '#idChkVeg4':{change:this.toggleVegCatBCBA},
      '#idChkVeg5':{change:this.toggleVegCatCCBA},
      '#idChkVeg6':{change:this.toggleVegCatDESA},
      '#idChkVeg7':{change:this.toggleVegCatDESAEquiv},
      '#idChkVeg8':{change:this.toggleVegetation},
      '#idChkVeg9':{change:this.toggleRoads},
      '#idChkSE1':{change:this.toggleMining},
      '#idChkSE2':{change:this.togglePASA},
      '#idChkSE3':{change:this.toggleSEAstronomy},
      '#idChkSE4':{change:this.toggleGroundwaterCA},
      '#idChkSE5':{change:this.toggleRoads},
      '#idChkSei1':{change:this.toggleMining},
      '#idChkSei2':{change:this.togglePASA},
      '#idChkSei3':{change:this.toggleGeolStructures},
      '#idChkSei4':{change:this.toggleSEIDensity},
      '#idChkSei5':{change:this.toggleRoads},
    });
  },
  changeVariable:function(field,newValue,oldValue,eOpts) {
    // Set the variable-specific parameters
    switch(newValue) {
      case 'Groundwater':
        // Show only the relevant panel
        Ext.ComponentQuery.query('#idPanelGW')[0].show();
        Ext.ComponentQuery.query('#idPanelSW')[0].hide();
        Ext.ComponentQuery.query('#idPanelVeg')[0].hide();
        Ext.ComponentQuery.query('#idPanelSE')[0].hide();
        Ext.ComponentQuery.query('#idPanelSei')[0].hide();
        // Turn on the default base layer
        this.layersReset();
        layerGWDrastic.setVisibility(true);
        layerQuaternary.setVisibility(true);
        Ext.ComponentQuery.query('#idRadGW1')[0].setValue(true);
        // Only activate the Boreholes radio button if within scale
        if (map.getScale() > 3000000){
          Ext.ComponentQuery.query('#idChkGW8')[0].disable();
        } else {
          Ext.ComponentQuery.query('#idChkGW8')[0].enable();
        }
        // Show the Information panel
        document.getElementById('infoGW').style.display = 'block';
        document.getElementById('infoSW').style.display = 'none';
        document.getElementById('infoVeg').style.display = 'none';
        document.getElementById('infoSE').style.display = 'none';
        document.getElementById('infoSei').style.display = 'none';
        // Show the Zoom to Quaternary dropdown and legend
        Ext.ComponentQuery.query('#idZoomQuat')[0].show();
        Ext.ComponentQuery.query('#idLegendQuat')[0].show();
        // Show the Zoom to WMA dropdown and legend
        Ext.ComponentQuery.query('#idZoomWMA')[0].show();
        Ext.ComponentQuery.query('#idLegendWMA')[0].show();
        // Hide the Uncategorised legend
        Ext.ComponentQuery.query('#idLegendUncat')[0].hide();
        break;
      case 'Surface Water':
        // Show only the relevant panel
        Ext.ComponentQuery.query('#idPanelGW')[0].hide();
        Ext.ComponentQuery.query('#idPanelSW')[0].show();
        Ext.ComponentQuery.query('#idPanelVeg')[0].hide();
        Ext.ComponentQuery.query('#idPanelSE')[0].hide();
        Ext.ComponentQuery.query('#idPanelSei')[0].hide();
        // Turn on the default base layer
        this.layersReset();
        layerSWDEC.setVisibility(true);
        layerQuaternary.setVisibility(true);
        Ext.ComponentQuery.query('#idRadSW1')[0].setValue(true);
        // Update the contents of the Information panel
        // Show the Information panel
        document.getElementById('infoGW').style.display = 'none';
        document.getElementById('infoSW').style.display = 'block';
        document.getElementById('infoVeg').style.display = 'none';
        document.getElementById('infoSE').style.display = 'none';
        document.getElementById('infoSei').style.display = 'none';
        // Show the Zoom to Quaternary dropdown and legend
        Ext.ComponentQuery.query('#idZoomQuat')[0].show();
        Ext.ComponentQuery.query('#idLegendQuat')[0].show();
        // Show the Zoom to WMA dropdown and legend
        Ext.ComponentQuery.query('#idZoomWMA')[0].show();
        Ext.ComponentQuery.query('#idLegendWMA')[0].show();
        // Show the Uncategorised legend
        Ext.ComponentQuery.query('#idLegendUncat')[0].show();
        break;
      case 'Vegetation':
        // Show only the relevant panel
        Ext.ComponentQuery.query('#idPanelGW')[0].hide();
        Ext.ComponentQuery.query('#idPanelSW')[0].hide();
        Ext.ComponentQuery.query('#idPanelVeg')[0].show();
        Ext.ComponentQuery.query('#idPanelSE')[0].hide();
        Ext.ComponentQuery.query('#idPanelSei')[0].hide();
        // Turn on the default base layer
        this.layersReset();
        layerVegAggr.setVisibility(true);
        Ext.ComponentQuery.query('#idRadVeg1')[0].setValue(true);
        // Show the Information panel
        document.getElementById('infoGW').style.display = 'none';
        document.getElementById('infoSW').style.display = 'none';
        document.getElementById('infoVeg').style.display = 'block';
        document.getElementById('infoSE').style.display = 'none';
        document.getElementById('infoSei').style.display = 'none';
        // Hide the Zoom to Quaternary dropdown and legend
        Ext.ComponentQuery.query('#idZoomQuat')[0].hide();
        Ext.ComponentQuery.query('#idLegendQuat')[0].hide();
        // Hide the Zoom to WMA dropdown and legend
        Ext.ComponentQuery.query('#idZoomWMA')[0].hide();
        Ext.ComponentQuery.query('#idLegendWMA')[0].hide();
        // Hide the Uncategorised legend
        Ext.ComponentQuery.query('#idLegendUncat')[0].hide();
        break;
      case 'Socio-Economics':
        // Show only the relevant panel
        Ext.ComponentQuery.query('#idPanelGW')[0].hide();
        Ext.ComponentQuery.query('#idPanelSW')[0].hide();
        Ext.ComponentQuery.query('#idPanelVeg')[0].hide();
        Ext.ComponentQuery.query('#idPanelSE')[0].show();
        Ext.ComponentQuery.query('#idPanelSei')[0].hide();
        // Turn on the default base layer
        this.layersReset();
        layerSEAggr.setVisibility(true);
        Ext.ComponentQuery.query('#idRadSE1')[0].setValue(true);
        // Show the Information panel
        document.getElementById('infoGW').style.display = 'none';
        document.getElementById('infoSW').style.display = 'none';
        document.getElementById('infoVeg').style.display = 'none';
        document.getElementById('infoSE').style.display = 'block';
        document.getElementById('infoSei').style.display = 'none';
        // Hide the Zoom to Quaternary dropdown and legend
        Ext.ComponentQuery.query('#idZoomQuat')[0].hide();
        Ext.ComponentQuery.query('#idLegendQuat')[0].hide();
        // Hide the Zoom to WMA dropdown and legend
        Ext.ComponentQuery.query('#idZoomWMA')[0].hide();
        Ext.ComponentQuery.query('#idLegendWMA')[0].hide();
        // Hide the Uncategorised legend
        Ext.ComponentQuery.query('#idLegendUncat')[0].hide();
        break;
      case 'Seismicity':
        // Show only the relevant panel
        Ext.ComponentQuery.query('#idPanelGW')[0].hide();
        Ext.ComponentQuery.query('#idPanelSW')[0].hide();
        Ext.ComponentQuery.query('#idPanelVeg')[0].hide();
        Ext.ComponentQuery.query('#idPanelSE')[0].hide();
        Ext.ComponentQuery.query('#idPanelSei')[0].show();
        // Turn on the default base layer
        this.layersReset();
        layerSeismicity.setVisibility(true);
        Ext.ComponentQuery.query('#idRadSei1')[0].setValue(true);
        // Show the Information panel
        document.getElementById('infoGW').style.display = 'none';
        document.getElementById('infoSW').style.display = 'none';
        document.getElementById('infoVeg').style.display = 'none';
        document.getElementById('infoSE').style.display = 'none';
        document.getElementById('infoSei').style.display = 'block';
        // Hide the Zoom to Quaternary dropdown and legend
        Ext.ComponentQuery.query('#idZoomQuat')[0].hide();
        Ext.ComponentQuery.query('#idLegendQuat')[0].hide();
        // Hide the Zoom to WMA dropdown and legend
        Ext.ComponentQuery.query('#idZoomWMA')[0].hide();
        Ext.ComponentQuery.query('#idLegendWMA')[0].hide();
        // Hide the Uncategorised legend
        Ext.ComponentQuery.query('#idLegendUncat')[0].hide();
        break;
    }
  },
  toggleSWDEC:function(field,newValue,oldValue,eOpts) {
    if (newValue){
      layerSWDEC.setVisibility(true);
      layerSWWetlRank.setVisibility(false);
    };
  },
  toggleSWWetlandRank:function(field,newValue,oldValue,eOpts) {
    if (newValue){
      layerSWDEC.setVisibility(false);
      layerSWWetlRank.setVisibility(true);
    };
  },
  toggleVegAggr:function(field,newValue,oldValue,eOpts) {
    if (newValue){
      layerVegAggr.setVisibility(true);
      layerVegETS.setVisibility(false);
      layerVegEPL.setVisibility(false);
    };
  },
  toggleVegETS:function(field,newValue,oldValue,eOpts) {
    if (newValue){
      layerVegAggr.setVisibility(false);
      layerVegETS.setVisibility(true);
      layerVegEPL.setVisibility(false);
    };
  },
  toggleVegEPL:function(field,newValue,oldValue,eOpts) {
    if (newValue){
      layerVegAggr.setVisibility(false);
      layerVegETS.setVisibility(false);
      layerVegEPL.setVisibility(true);
    };
  },
  toggleSEAggr:function(field,newValue,oldValue,eOpts) {
    if (newValue){
      layerSEAggr.setVisibility(true);
      layerSEDensity.setVisibility(false);
      layerSEChildren.setVisibility(false);
      layerSEGroundwater.setVisibility(false);
      layerSEEmployment.setVisibility(false);
      layerSEFemaleHH.setVisibility(false);
    };
  },
  toggleSEDensity:function(field,newValue,oldValue,eOpts) {
    if (newValue){
      layerSEAggr.setVisibility(false);
      layerSEDensity.setVisibility(true);
      layerSEChildren.setVisibility(false);
      layerSEGroundwater.setVisibility(false);
      layerSEEmployment.setVisibility(false);
      layerSEFemaleHH.setVisibility(false);
    };
  },
  toggleSEChildren:function(field,newValue,oldValue,eOpts) {
    if (newValue){
      layerSEAggr.setVisibility(false);
      layerSEDensity.setVisibility(false);
      layerSEChildren.setVisibility(true);
      layerSEGroundwater.setVisibility(false);
      layerSEEmployment.setVisibility(false);
      layerSEFemaleHH.setVisibility(false);
    };
  },
  toggleSEGroundwater:function(field,newValue,oldValue,eOpts) {
    if (newValue){
      layerSEAggr.setVisibility(false);
      layerSEDensity.setVisibility(false);
      layerSEChildren.setVisibility(false);
      layerSEGroundwater.setVisibility(true);
      layerSEEmployment.setVisibility(false);
      layerSEFemaleHH.setVisibility(false);
    };
  },
  toggleSEEmployment:function(field,newValue,oldValue,eOpts) {
    if (newValue){
      layerSEAggr.setVisibility(false);
      layerSEDensity.setVisibility(false);
      layerSEChildren.setVisibility(false);
      layerSEGroundwater.setVisibility(false);
      layerSEEmployment.setVisibility(true);
      layerSEFemaleHH.setVisibility(false);
    };
  },
  toggleSEFemaleHH:function(field,newValue,oldValue,eOpts) {
    if (newValue){
      layerSEAggr.setVisibility(false);
      layerSEDensity.setVisibility(false);
      layerSEChildren.setVisibility(true);
      layerSEGroundwater.setVisibility(false);
      layerSEEmployment.setVisibility(false);
      layerSEFemaleHH.setVisibility(true);
    };
  },
  toggleMining:function(field,newValue,oldValue,eOpts) {
    layerMiningProhibited.setVisibility(newValue);
  },
  togglePASA:function(field,newValue,oldValue,eOpts) {
    layerPASA.setVisibility(newValue);
  },
  toggleGeolStructures:function(field,newValue,oldValue,eOpts) {
    layerGeolStructures.setVisibility(newValue);
    layerGeolStructuresSills.setVisibility(newValue);
    layerSprings.setVisibility(newValue);
  },
  toggleRivers:function(field,newValue,oldValue,eOpts) {
    layerRivers.setVisibility(newValue);
  },
  toggleWMA:function(field,newValue,oldValue,eOpts) {
    layerWMA.setVisibility(newValue);
  },
  toggleBoreholes:function(field,newValue,oldValue,eOpts) {
    layerBoreholes.setVisibility(newValue);
  },
  toggleSWWetlClusters:function(field,newValue,oldValue,eOpts) {
    layerSWWetlClusters.setVisibility(newValue);
  },
  toggleSWSWSABlobs:function(field,newValue,oldValue,eOpts) {
    layerSWSWSABlobs.setVisibility(newValue);
  },
  toggleSWFish:function(field,newValue,oldValue,eOpts) {
    layerSWFish.setVisibility(newValue);
  },
  toggleVegADE:function(field,newValue,oldValue,eOpts) {
    layerVegADE.setVisibility(newValue);
  },
  toggleVegCatBCBA:function(field,newValue,oldValue,eOpts) {
    layerVegCatBCBA.setVisibility(newValue);
  },
  toggleVegCatCCBA:function(field,newValue,oldValue,eOpts) {
    layerVegCatCCBA.setVisibility(newValue);
  },
  toggleVegCatDESA:function(field,newValue,oldValue,eOpts) {
    layerVegCatDESA.setVisibility(newValue);
  },
  toggleVegCatDESAEquiv:function(field,newValue,oldValue,eOpts) {
    layerVegCatDESAEquiv.setVisibility(newValue);
  },
  toggleVegetation:function(field,newValue,oldValue,eOpts) {
    layerVegetation.setVisibility(newValue);
  },
  toggleSEAstronomy:function(field,newValue,oldValue,eOpts) {
    layerSEAstronomy.setVisibility(newValue);
    layerSEAstronomyMeerkat.setVisibility(newValue);
  },
  toggleGroundwaterCA:function(field,newValue,oldValue,eOpts) {
    layerGroundwaterCA.setVisibility(newValue);
  },
  toggleVegter:function(field,newValue,oldValue,eOpts) {
    layerGWVegter.setVisibility(newValue);
  },
  toggleSEIDensity:function(field,newValue,oldValue,eOpts) {
    layerSEIDensity.setVisibility(newValue);
  },
  toggleRoads:function(field,newValue,oldValue,eOpts) {
    layerRoads.setVisibility(newValue);
  },
  layersReset:function(){
  /* This functions turns all layers off apart from the base and border layers.
  */

    // Loop through the layers and turn them off
    var layerArray = map.layers;
    for (var i=0;i<layerArray.length;i++) {
      map.layers[i].setVisibility(false);
    };

    // Turn on the base and borders layers
    layerBase.setVisibility(true);
    layerOverlays.setVisibility(true);

    // Untick all the check boxes
    Ext.ComponentQuery.query('#idChkGW1')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkGW2')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkGW3')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkGW4')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkGW5')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkGW6')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkGW7')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkGW8')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkGW9')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkSW1')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkSW2')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkSW3')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkSW4')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkSW5')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkSW6')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkSW7')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkSW8')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkVeg1')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkVeg2')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkVeg3')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkVeg4')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkVeg5')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkVeg6')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkVeg7')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkVeg8')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkVeg9')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkSE1')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkSE2')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkSE3')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkSE4')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkSE5')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkSei1')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkSei2')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkSei3')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkSei4')[0].setValue(false);
    Ext.ComponentQuery.query('#idChkSei5')[0].setValue(false);
  },
});
