/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * A copy of the GNU General Public License is included along with this
 * program in the file gpl-3.0.txt.
 *
 */
var map;
var panelMap;
var panelLayers;
var layerBase;
var layerOverlays;
var layerMiningProhibited;
var layerPASA;
var layerGeolStructures;
var layerGeolStructuresSills;
var layerSprings;
var layerBoreholes;
var layerQuaternary;
var layerWMA;
var layerGroundwaterCA;
var layerGWDrastic;
var layerGWVegter;
var layerRivers;
var layerRoads;
var layerSWDEC;
var layerSWWetlRank;
var layerSWSWSABlobs;
var layerSWWetlClusters;
var layerSWFish;
var layerVegAggr;
var layerVegETS;
var layerVegEPL;
var layerVegADE;
var layerVegCatBCBA;
var layerVegCatCCBA;
var layerVegCatDESA;
var layerVegCatDESAEquiv;
var layerVegetation;
var layerSEAggr;
var layerSEDensity;
var layerSEChildren;
var layerSEGroundwater;
var layerSEEmployment;
var layerSEFemaleHH;
var layerSEAstronomy;
var layerSEAstronomyMeerkat;
var layerSeismicity;
var layerSEIDensity;
var bounds;
var options;
var mapExtentX = 24.6716;
var mapExtentY = -28.4796;
var mapExtentZoom = 13;
var proj4326 = new OpenLayers.Projection('EPSG:4326');
var popupControl;
var wmsServerURL = "/cgi-bin/mapserv?map=mapfile&";

function zoomFull() {
/* This function zooms the map to its full extent.
*/
  map.zoomToExtent(bounds);
}

// Define a control to handle user clicks on the map
OpenLayers.Control.Click = OpenLayers.Class(OpenLayers.Control, {
  defaultHandlerOptions:{
    single:true,
    double:false,
    pixelTolerance:0,
    stopSingle:true,
  },
  initialize:function(options){
    this.handlerOptions = OpenLayers.Util.extend(
      options && options.handlerOptions || {},
      this.defaultHandlerOptions
    );
    OpenLayers.Control.prototype.initialize.apply(
      this, arguments
    );
    this.handler = new OpenLayers.Handler.Click(
      this,
      {
        click:this.trigger
      },
      this.handlerOptions
    );
  },
  CLASS_NAME:"OpenLayers.Control.Click",
});

function getFeatureInfoParams(x,y,format,queryLayer){
/* This function returns a string containing parameters for a
   WMSGetFeatureInfo request.
*/
  var params = 'REQUEST=GetFeatureInfo'
    + '&SERVICE=WMS'
    + '&VERSION=1.1.1'
    + '&EXCEPTIONS=application/vnd.ogc.se_xml'
    + '&BBOX=' + map.getExtent().toBBOX()
    + '&X=' + Math.round(x)
    + '&Y=' + Math.round(y)
    + '&INFO_FORMAT='+format
    + '&QUERY_LAYERS='+queryLayer
    + '&LAYERS='+queryLayer
    + '&FEATURE_COUNT=100'
    + '&SRS=EPSG:4326'
    + '&STYLES='
    + '&WIDTH=' + map.size.w
    + '&HEIGHT=' + map.size.h;
  return params;
};

function enableBoreholes(event){
  if (map.getScale() > 3000000){
    Ext.ComponentQuery.query('#idChkGW8')[0].disable();
  } else {
    Ext.ComponentQuery.query('#idChkGW8')[0].enable();
  }
};

Ext.require([
  'Ext.container.Viewport',
  'Ext.layout.container.Border',
  'Ext.window',
]);

Ext.application({
  name:'Fracking',
  appFolder:'/js',
  controllers:['MapLayers','ZoomLayers','Popup'],
  stores:['Quaternaries','WMAs','Towns','Fish','SQFish'],
  launch:function() {
    _frackingApp = this;

    // Define the bounds for the map
    bounds = new OpenLayers.Bounds(16.23,-35.2,33.10,-21.9);

    // Define the options for the map
    options = {
      projection:proj4326,
      displayProjection:proj4326,
      units:'dd',
      maxResolution:0.0225,
      maxExtent:bounds,
      restrictedExtent:bounds,
      scales:[7087101.48,4250000,2550000,1530000,920000,550000,330000,200000,120000],
      eventListeners:{'zoomend':enableBoreholes},
    };

    // Define the map
    map = new OpenLayers.Map('map', options);

    // Add a loading indicator
    map.addControl(new OpenLayers.Control.LoadingPanel());

    // Define the map base layer
    layerBase = new OpenLayers.Layer.WMS(
      "Base",
      wmsServerURL,
      {layers:'Base',format:'image/png'},
      {isbaseLayer:true,visibility:true,displayInLayerSwitcher:false}
    );
    map.addLayer(layerBase);

    // Define the map borders layer
    layerOverlays = new OpenLayers.Layer.WMS(
      "Overlays",
      wmsServerURL,
      {layers:'Overlays',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:true,displayInLayerSwitcher:false,singleTile:true}
    );

    // Define the theme base layers
    layerSWDEC = new OpenLayers.Layer.WMS(
      "Default Ecological Category",
      wmsServerURL,
      {layers:'DEC',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:true,displayInLayerSwitcher:false,singleTile:false}
    );
    layerSWWetlRank = new OpenLayers.Layer.WMS(
      "Wetland Rank",
      wmsServerURL,
      {layers:'WetlandRank',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerGWDrastic = new OpenLayers.Layer.WMS(
      "Groundwater DRASTIC",
      wmsServerURL,
      {layers:'Drastic',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerVegAggr = new OpenLayers.Layer.WMS(
      "Vegetation Aggregated",
      wmsServerURL,
      {layers:'VegetationAggregated',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerVegETS = new OpenLayers.Layer.WMS(
      "Vegetation Ecosystem Threat Status",
      wmsServerURL,
      {layers:'ETS',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerVegEPL = new OpenLayers.Layer.WMS(
      "Vegetation Ecosystem Protection Level",
      wmsServerURL,
      {layers:'PLS',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerSeismicity = new OpenLayers.Layer.WMS(
      "layerSeismicity",
      wmsServerURL,
      {layers:'Seismicity',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerSEAggr = new OpenLayers.Layer.WMS(
      "Aggregated Socio-Economics",
      wmsServerURL,
      {layers:'SocioEconomicsAggregated',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerSEDensity = new OpenLayers.Layer.WMS(
      "Population Density",
      wmsServerURL,
      {layers:'PopulationDensity',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerSEChildren = new OpenLayers.Layer.WMS(
      "Children under 5",
      wmsServerURL,
      {layers:'Children',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerSEGroundwater = new OpenLayers.Layer.WMS(
      "Population Dependent on Groundwater",
      wmsServerURL,
      {layers:'Groundwater',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerSEEmployment = new OpenLayers.Layer.WMS(
      "Population Employed by Agriculture",
      wmsServerURL,
      {layers:'Agriculture',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerSEFemaleHH = new OpenLayers.Layer.WMS(
      "Female Headed Households",
      wmsServerURL,
      {layers:'FemaleHH',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    map.addLayers([layerGWDrastic,layerSWDEC,layerSWWetlRank,layerVegAggr,layerVegETS,layerVegEPL,
                   layerSEAggr,layerSEDensity,layerSEChildren,layerSEGroundwater,layerSEEmployment,
                   layerSEFemaleHH,layerSeismicity]);

    // Define the overlay layers
    layerMiningProhibited = new OpenLayers.Layer.WMS(
      "Mining Legally Prohibited",
      wmsServerURL,
      {layers:'MiningProhibited',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerPASA = new OpenLayers.Layer.WMS(
      "PASA Permit Areas",
      wmsServerURL,
      {layers:'PASA',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerGeolStructures = new OpenLayers.Layer.WMS(
      "Geological Structures",
      wmsServerURL,
      {layers:'GeologicalStructures',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerGeolStructuresSills = new OpenLayers.Layer.WMS(
      "Geological Structures - Sills",
      wmsServerURL,
      {layers:'GeologicalStructuresSills',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerRivers = new OpenLayers.Layer.WMS(
      "Rivers",
      wmsServerURL,
      {layers:'Rivers',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerRoads = new OpenLayers.Layer.WMS(
      "Roads",
      wmsServerURL,
      {layers:'Roads',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerSprings = new OpenLayers.Layer.WMS(
      "Springs",
      wmsServerURL,
      {layers:'Springs',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerBoreholes = new OpenLayers.Layer.WMS(
      "Boreholes",
      wmsServerURL,
      {layers:'Boreholes',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerQuaternary = new OpenLayers.Layer.WMS(
      "Quaternary Catchments",
      wmsServerURL,
      {layers:'Quaternary',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:true,displayInLayerSwitcher:false,singleTile:true}
    );
    layerWMA = new OpenLayers.Layer.WMS(
      "Water Management Areas",
      wmsServerURL,
      {layers:'WMA',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:true}
    );
    layerSWWetlClusters = new OpenLayers.Layer.WMS(
      "Wetland Clusters",
      wmsServerURL,
      {layers:'WetlandClusters',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerSWSWSABlobs = new OpenLayers.Layer.WMS(
      "SWSA Blobs Overlay",
      wmsServerURL,
      {layers:'SWSABlobs',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerSWFish = new OpenLayers.Layer.WMS(
      "Threatened fish species",
      wmsServerURL,
      {layers:'Fish',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerVegADE = new OpenLayers.Layer.WMS(
      "Aquifer Dependent Ecosystems",
      wmsServerURL,
      {layers:'ADE',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerVegCatBCBA = new OpenLayers.Layer.WMS(
      "Category B Critical Biodiversity Area",
      wmsServerURL,
      {layers:'CatBCBA',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerVegCatCCBA = new OpenLayers.Layer.WMS(
      "Category C Critical Biodiversity Area",
      wmsServerURL,
      {layers:'CatCCBA',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerVegCatDESA = new OpenLayers.Layer.WMS(
      "Category D Ecological Support Area",
      wmsServerURL,
      {layers:'CatDESA',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerVegCatDESAEquiv = new OpenLayers.Layer.WMS(
      "Category D Ecological Support Area Equivalents",
      wmsServerURL,
      {layers:'CatDESAEquiv',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerVegetation = new OpenLayers.Layer.WMS(
      "Vegetation of South Africa",
      wmsServerURL,
      {layers:'Vegetation',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerSEAstronomy = new OpenLayers.Layer.WMS(
      "Astronomy Assessment Areas",
      wmsServerURL,
      {layers:'Astronomy',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerSEAstronomyMeerkat = new OpenLayers.Layer.WMS(
      "Astronomy Meerkat",
      wmsServerURL,
      {layers:'AstronomyMeerkat',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerGroundwaterCA = new OpenLayers.Layer.WMS(
      "Groundwater Control Areas",
      wmsServerURL,
      {layers:'GroundwaterCA',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerGWVegter = new OpenLayers.Layer.WMS(
      "Vegters Groundwater Regions",
      wmsServerURL,
      {layers:'GWVegter',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );
    layerSEIDensity = new OpenLayers.Layer.WMS(
      "Population Density",
      wmsServerURL,
      {layers:'PopDensityOverlay',format:'image/png',transparent:true },
      {isbaseLayer:false,visibility:false,displayInLayerSwitcher:false,singleTile:false}
    );

    // Add the layers to the map
    map.addLayers([layerMiningProhibited,layerPASA,layerGeolStructuresSills,
                   layerGeolStructures,layerSprings,layerBoreholes,layerQuaternary,
                   layerWMA,layerRivers,layerRoads,layerSWWetlClusters,layerSWSWSABlobs,
                   layerSWFish,layerVegADE,layerVegCatBCBA,layerVegCatCCBA,
                   layerVegCatDESA,layerVegCatDESAEquiv,layerVegetation,
                   layerSEAstronomy,layerSEAstronomyMeerkat,layerGroundwaterCA,
                   layerGWVegter,layerSEIDensity,layerOverlays]);

    // Setup the scale bar
    map.addControl(new OpenLayers.Control.ScaleLine({
      bottomInUnits:'',
      bottomOutUnits:'',
    }));

    // Setup the map panel
    panelMap = Ext.create('Ext.panel.Panel', {
      region:'center',
      width:750,
      height:600,
      contentEl:'map',
    });
    zoomFull();

    // Define the layer panel
    panelLayers = Ext.create('Ext.panel.Panel', {
      region:'west',
      title:'Vulnerability Map Themes',
      width:200,
      height:600,
      layout:'absolute',
      items:[
        {
          xtype:'maplayers',
          x:0,
          y:0,
          height:490,
        },{
          xtype:'zoomlayers',
          x:0,
          y:495,
          height:78,
        },
      ],
    });

    // Define the map application panel
    Ext.create('Ext.panel.Panel', {
      layout:'border',
      renderTo:'mapView',
      width:950,
      height:600,
      minHeight:600,
      items:[
        panelMap,
        panelLayers,
      ],
    });

    // Define the feature popup control
    popupControl = new OpenLayers.Control.Click({
      trigger:function(e) {
        // Work out which layers to request
        var queryLayers = '';
        var theme = Ext.ComponentQuery.query('#idVariable')[0].getRawValue();
        switch(theme){
          case 'Groundwater':
            if (Ext.ComponentQuery.query('#idChkGW1')[0].checked) queryLayers += 'MiningProhibited'+',';
            if (Ext.ComponentQuery.query('#idChkGW2')[0].checked) queryLayers += 'PASA'+',';
            if (Ext.ComponentQuery.query('#idChkGW3')[0].checked) queryLayers += 'GeologicalStructures,Springs'+',';
            if (Ext.ComponentQuery.query('#idChkGW4')[0].checked) queryLayers += 'GroundwaterCA'+',';
            if (Ext.ComponentQuery.query('#idChkGW5')[0].checked) queryLayers += 'GWVegter'+',';
            if (Ext.ComponentQuery.query('#idChkGW6')[0].checked) queryLayers += 'Rivers'+',';
            if (Ext.ComponentQuery.query('#idChkGW7')[0].checked) queryLayers += 'WMA'+',';
            if (Ext.ComponentQuery.query('#idChkGW8')[0].checked) queryLayers += 'Boreholes'+',';
            break;
          case 'Surface Water':
            if (Ext.ComponentQuery.query('#idRadSW1')[0].checked) queryLayers += 'DEC'+',';
            if (Ext.ComponentQuery.query('#idRadSW2')[0].checked) queryLayers += 'WetlandRank'+',';
            if (Ext.ComponentQuery.query('#idChkSW1')[0].checked) queryLayers += 'MiningProhibited'+',';
            if (Ext.ComponentQuery.query('#idChkSW2')[0].checked) queryLayers += 'PASA'+',';
            if (Ext.ComponentQuery.query('#idChkSW3')[0].checked) queryLayers += 'WetlandClusters'+',';
            if (Ext.ComponentQuery.query('#idChkSW4')[0].checked) queryLayers += 'SWSABlobs'+',';
            if (Ext.ComponentQuery.query('#idChkSW5')[0].checked) queryLayers += 'Fish'+',';
            if (Ext.ComponentQuery.query('#idChkSW6')[0].checked) queryLayers += 'Rivers'+',';
            if (Ext.ComponentQuery.query('#idChkSW7')[0].checked) queryLayers += 'WMA'+',';
            break;
          case 'Vegetation':
            if (Ext.ComponentQuery.query('#idChkVeg1')[0].checked) queryLayers += 'MiningProhibited'+',';
            if (Ext.ComponentQuery.query('#idChkVeg2')[0].checked) queryLayers += 'PASA'+',';
            if (Ext.ComponentQuery.query('#idChkVeg3')[0].checked) queryLayers += 'ADE'+',';
            if (Ext.ComponentQuery.query('#idChkVeg4')[0].checked) queryLayers += 'CatBCBA'+',';
            if (Ext.ComponentQuery.query('#idChkVeg5')[0].checked) queryLayers += 'CatCCBA'+',';
            if (Ext.ComponentQuery.query('#idChkVeg6')[0].checked) queryLayers += 'CatDESA'+',';
            if (Ext.ComponentQuery.query('#idChkVeg7')[0].checked) queryLayers += 'CatDESAEquiv'+',';
            if (Ext.ComponentQuery.query('#idChkVeg8')[0].checked) queryLayers += 'Vegetation'+',';
            break;
          case 'Socio-Economics':
            if (Ext.ComponentQuery.query('#idChkSE1')[0].checked) queryLayers += 'MiningProhibited'+',';
            if (Ext.ComponentQuery.query('#idChkSE2')[0].checked) queryLayers += 'PASA'+',';
            if (Ext.ComponentQuery.query('#idChkSE3')[0].checked) queryLayers += 'Astronomy,AstronomyMeerkat'+',';
            if (Ext.ComponentQuery.query('#idChkSE4')[0].checked) queryLayers += 'GroundwaterCA'+',';
            break;
          case 'Seismicity':
            if (Ext.ComponentQuery.query('#idChkSei1')[0].checked) queryLayers += 'MiningProhibited'+',';
            if (Ext.ComponentQuery.query('#idChkSei2')[0].checked) queryLayers += 'PASA'+',';
            if (Ext.ComponentQuery.query('#idChkSei3')[0].checked) queryLayers += 'GeologicalStructures,Springs'+',';
            if (Ext.ComponentQuery.query('#idChkSei4')[0].checked) queryLayers += 'PopDensityOverlay'+',';
            break;
        };

        // Send the WMS GetFeatureInfo request if any of the overlays are active
        if (queryLayers != ''){
          // Remove the trailing comma
          queryLayers = queryLayers.substring(0, queryLayers.length - 1);
          // Setup and send the request
          var WMSParams = getFeatureInfoParams(e.xy.x,e.xy.y,'text/html',queryLayers);
          Ext.Ajax.request({
            url:wmsServerURL+WMSParams,
            success:function(response,opts){
              if (response.responseText.length > 4){
                _frackingApp.getController('Popup').addContent(response.responseText);
              };
            },
            failure:function(response,opts){
              // Fail silently
            }
          });
        };
      }
    });
    map.addControl(popupControl);
    popupControl.activate();

  },
});

